import React, { useState } from 'react';
import { pillWidth, pillHeight, pillXPadding, pillInnerMargin, topLineHeight, pillBorderWidth, pillDropdownPadding } from './pill.const';
import { Position } from 'reactflow';
import { PillHandle } from './PillHandle';

export type PillI = {
  onHovered?: (hovered: boolean) => void;
  // onDragged: (dragged: boolean) => void;
  title: string | React.ReactNode;
  children?: React.ReactNode;
  handles?: React.ReactNode;
  style?: React.CSSProperties;
  corner?: 'rounded' | 'square' | 'diamond';
  topColor: string;
  handleUp?: React.ReactNode;
  handleDown?: React.ReactNode;
  handleLeft?: React.ReactNode;
  handleRight?: React.ReactNode;
  className?: string;
};

export const Pill = React.memo((props: PillI) => {
  const [hovered, setHovered] = useState(false);

  const onMouseEnter = (event: React.MouseEvent) => {
    if (!hovered) {
      setHovered(true);
      if (props.onHovered) props.onHovered(true);
    }
  };

  const onMouseLeave = (event: React.MouseEvent) => {
    if (hovered) {
      setHovered(false);
      if (props.onHovered) props.onHovered(false);
    }
  };

  const width = pillWidth;
  const corner = props.corner || 'rounded';

  const innerContentStyle = {
    minWidth: corner === 'diamond' ? width - pillBorderWidth * 2 : width - pillBorderWidth * 2,
    maxWidth: corner === 'diamond' ? width - pillBorderWidth * 2 : width - pillBorderWidth * 2,
    minHeight: pillHeight - pillBorderWidth * 2,
    maxHeight: pillHeight - pillBorderWidth * 2,
    margin: pillBorderWidth,
    clipPath: corner === 'diamond' ? 'polygon(0.5% 50%, 4.9% 0%, 95.1% 0%, 99.5% 50%, 95.1% 100%, 4.9% 100%)' : '',
  };

  const outerContentStyle = {
    minWidth: width,
    maxWidth: width,
    minHeight: pillHeight,
    maxHeight: pillHeight,
    clipPath: corner === 'diamond' ? 'polygon(0% 50%, 5% 0%, 95% 0%, 100% 50%, 95% 100%, 5% 100%)' : '',
  };

  const onDragStart = (event: React.DragEvent<HTMLDivElement>) => {
    const pill = event.target as HTMLDivElement;
    const dragImage = pill.cloneNode(true) as HTMLDivElement;
    dragImage.style.transform = 'translate(0, 0)';  // Removes the background from drag image in Chrome
    document.body.appendChild(dragImage);

    const mouse_x = event.clientX - pill.getBoundingClientRect().left;
    const mouse_y = event.clientY - pill.getBoundingClientRect().top;

    event.dataTransfer.setDragImage(dragImage, mouse_x, mouse_y);
    event.dataTransfer.setData('mouse_x', String(mouse_x))
    event.dataTransfer.setData('mouse_y', String(mouse_y))
    
    setTimeout(() => {
      dragImage.remove();
    }, 1);
  }

  return (
    <div draggable="true" onDragStart={onDragStart} className={(props.className ? props.className + ' ' : '') + 'flex flex-row flex-grow-0 text-xs text-justify'}>
      <div
        className={'bg-secondary-200 ' + (corner !== 'square' ? 'rounded' : '')}
        style={{
          ...outerContentStyle,
        }}
      >
        <div
          className="absolute -z-10"
          style={{
            top: pillHeight - pillInnerMargin,
            width: pillWidth,
            paddingLeft: pillDropdownPadding,
            paddingRight: pillDropdownPadding,
          }}
        >
          {props.children}
        </div>
        <div
          className={'bg-secondary-100 ' + (corner !== 'square' ? 'rounded-[3px]' : '')}
          style={{
            ...innerContentStyle,
          }}
        >
          {props.topColor && (
            <div className={(corner !== 'square' ? 'rounded-t ' : '') + props.topColor} style={{ height: topLineHeight }}></div>
          )}
          <div
            className={'font-semibold bg-neutral-100 ' + (corner !== 'square' ? 'rounded-b-[3px]' : '')}
            onMouseEnter={onMouseEnter}
            onMouseLeave={onMouseLeave}
          >
            <div
              className=""
              style={{
                transform: `translateY(${topLineHeight / 3}px)`,
                paddingLeft: pillXPadding + (corner === 'diamond' ? 5 : 0),
                paddingRight: pillXPadding + (corner === 'diamond' ? 5 : 0),
              }}
            >
              {props.title}
            </div>
          </div>
        </div>
      </div>
      <div className="absolute z-50 pointer-events-auto">{props.handles}</div>
    </div>
  );
});

export const EntityPill = React.memo((props: Omit<PillI, 'topColor'> & { withHandles?: 'vertical' | 'horizontal' }) => {
  const handles = !props.withHandles ? undefined : props.withHandles === 'horizontal' ? (
    <>
      <PillHandle position={Position.Left} className={'fill-accent-500 stroke-white'} type="square">
        {props.handleLeft}
      </PillHandle>
      <PillHandle position={Position.Right} className={'fill-accent-500 stroke-white'} type="square">
        {props.handleRight}
      </PillHandle>
    </>
  ) : (
    <>
      <PillHandle position={Position.Top} className={'fill-accent-500 stroke-white stroke-1'} type="arrowUp">
        {props.handleUp}
      </PillHandle>
      <PillHandle position={Position.Bottom} className={'fill-accent-500 stroke-white stroke-1'} type="arrowDown">
        {props.handleDown}
      </PillHandle>
    </>
  );
  return <Pill {...props} corner="rounded" topColor="bg-accent-500" handles={handles} />;
});

export const RelationPill = React.memo((props: Omit<PillI, 'topColor'> & { withHandles?: 'vertical' | 'horizontal' }) => {
  const handles = !props.withHandles ? undefined : props.withHandles === 'horizontal' ? (
    <>
      <PillHandle position={Position.Left} className={'fill-[#0676C1] stroke-white'} type="square">
        {props.handleLeft}
      </PillHandle>
      <PillHandle position={Position.Right} className={'fill-[#0676C1] stroke-white'} type="square" mr={-pillBorderWidth}>
        {props.handleRight}
      </PillHandle>
    </>
  ) : (
    <>
      <PillHandle position={Position.Top} className={'fill-[#0676C1] stroke-white stroke-1'} type="arrowUp">
        {props.handleUp}
      </PillHandle>
      <PillHandle position={Position.Bottom} className={'fill-[#0676C1] stroke-white stroke-1'} type="arrowDown">
        {props.handleDown}
      </PillHandle>
    </>
  );

  return <Pill {...props} corner="diamond" topColor="bg-[#0676C1]" handles={handles} />;
});

export const LogicPill = React.memo((props: Omit<PillI, 'topColor'>) => {
  const handles = (
    <PillHandle position={Position.Left} className={'fill-[#543719] stroke-white'} type="square">
      {props.handleLeft}
    </PillHandle>
  );

  return <Pill {...props} corner="square" topColor="bg-[#543719]" handles={handles} />;
});
